// ---------------------------------------------------------------------------------------------------------------------
// TESTS: Button callbacks in deeply nested forms, and validators on custom controls
// ---------------------------------------------------------------------------------------------------------------------

import { FormControl, Validators, ValidationErrors } from '@angular/forms';

// For the simple async validator
import { Observable, of } from 'rxjs';
import { delay, map } from 'rxjs/internal/operators';

const testAsyncValidator = (fc: FormControl): Observable<ValidationErrors> => {
	return of(fc).pipe(delay(2000)).pipe(map(_fc => {
		console.log('Async validator got', _fc.value);
		return _fc.value === '4200' ? null : { is4200: false };
	}));
};


const model = {};

const meta = {
	buttonGroup: { type: 'buttonGroup', label: 'Button Group',
		meta: [
			{ label: 'Say Hello', fnId: 'SAYHELLO' },
			{ label: 'Say Cheese', fnId: 'SAYCHEESE' }
		]
	},
	dynaformtest: {
		meta: {
			validatorTest: {
				validators: [ Validators.required, Validators.minLength(4) ],
				// perhaps have some standard messages for standard failures in the Object.prototype ??
				valFailureMsgs: {
					'required': 'This field is simply a MUST',
					'minlength': 'Please type something longer'
				}
			},
			validatorsOnCustomControl: {
				type: 'dropdownModifiedInput',
				validators: [ Validators.required, Validators.minLength(4) ],
				asyncValidators: testAsyncValidator
			},
			c: { label: 'Property Three', type: 'radio', options: ['Yes', 'No', 'Maybe'], horizontal: 1 },
			d: {
				meta: {
					isWorking: { type: 'radio', 'label': 'Does it work yet?', validators: Validators.required },
					f: { type: 'datepicker' },
					deeplyNestedButtonGroup: { type: 'buttonGroup', label: 'Deeply Nested Buttons',
						meta: [
							{ label: 'Say Hello', fnId: 'SAYHELLO' },
							{ label: 'Say Cheese', fnId: 'SAYCHEESE' }
						]
					}
				}
			}
		}
	}
};

export { model, meta };
