import { Component, Input, ChangeDetectorRef } from '@angular/core';
import { NativeInputComponent } from '../../_abstract/native-input.component';
import { Router, ActivatedRoute } from '@angular/router';
import { IOption, ILink } from '../../../models/field.model';
import { FriendlyValidationErrorsService } from './../../../services/friendly-validation-errors.service';


@Component({
	selector: 'app-select',
	templateUrl: './clr-select.component.html',
	styleUrls: ['./clr-select.component.scss']
})
export class ClrSelectComponent extends NativeInputComponent {

	@Input()
	set meta(meta: StringMap<any>) {
		// If there's only one choice set the FormControls value (otherwise the field shows the choice, but the FormControl's value remains unchanged,
		// maybe due to no updateOnBlur interaction?)
		// If there are multiple choices that don't include the current value prepend a blank option
		if (meta.options.length === 1) {
			this.control.setValue(meta.options[0].value);
			this.control.markAsDirty();
			this.control.markAsTouched();
			this.control.updateValueAndValidity();
		} else {
			const cval = this.control.value; // current value
			// tslint:disable-next-line: triple-equals
			const newOptsIncludeCVal = meta.options.find((opt: IOption) => opt.value == cval); // Deliberate type coercion with '=='
			if (!newOptsIncludeCVal) {
				meta.options.unshift({ label: '', value: ''});
				this.control.setValue('');
				this.control.markAsPristine();
				this.control.markAsUntouched();
				this.control.updateValueAndValidity();
			}
		}
		this._meta = meta;
		this.exposeForTemplate();
	}

	exposeMetaInTemplate: string[] = ['label', 'options', 'link'];

	label: string;
	options: IOption[];
	link: ILink;

	readonly componentName = 'SelectComponent'; // For AOT compatibility, as class names don't survive minification

	// TODO: Emit event to parant dynaform to do routing - then all form components will be able to use it. Much cleaner!
	constructor(
		private router: Router,
		private route: ActivatedRoute,
		protected valErrsService: FriendlyValidationErrorsService,
		protected _cdr: ChangeDetectorRef
	) {
		super(valErrsService, _cdr);
	}

	handleChange() {
		this.control.markAsDirty();
		this.control.markAsTouched();
		this.control.updateValueAndValidity();
		super.handleChange();
	}

	navigate(field: HTMLSelectElement) {
		const base = Array.isArray(this.meta.link.route) ? this.meta.link.route : [this.meta.link.route];
		const destination = [...base, field.options[field.selectedIndex].value];
		this.router.navigate(destination, { relativeTo: this.route });
	}

}
