import { Component, OnInit, forwardRef, ChangeDetectorRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { CustomInputComponent } from './../../_abstract/custom-input.component';
import { FriendlyValidationErrorsService } from './../../../services/friendly-validation-errors.service';


@Component({
	selector: 'amp-datetime',
	templateUrl: './datetime.component.html',
	styleUrls: ['./datetime.component.scss'],
	providers: [
		{
			provide: NG_VALUE_ACCESSOR,
			useExisting: forwardRef(() => DatetimeComponent),
			multi: true
		}
	]
})
export class DatetimeComponent extends CustomInputComponent implements OnInit {

	exposeMetaInTemplate: string[] = ['label'];

	label: string;

	value: Date;

	date: string;
	mm: string;
	hh: string;
	hourOptions: string[] = new Array(24).fill(0).map((m, i) => `0${i}`.slice(-2));
	minuteOptions: string[] = new Array(60).fill(0).map((m, i) => `0${i}`.slice(-2));

	readonly componentName = 'DatetimeComponent'; // For AOT compatibility, as class names don't survive minification

	constructor(
		protected valErrsService: FriendlyValidationErrorsService,
		protected _cdr: ChangeDetectorRef
	) {
		super(valErrsService, _cdr);
	}

	ngOnInit() {
		this.splitIntoDateAndTime(this.control.value);
	}

	writeValue(value: any): void {
		this.value = value;
		this.splitIntoDateAndTime(value);
	}

	trackByFn(index: any, item: any) {
		return index;
	}

	updateValue(id: 'date'|'hours'|'minutes', value: string): void {
		switch (id) {
			case 'date':
				this.date = value;
				break;
			case 'hours':
				this.hh = value;
				break;
			case 'minutes':
				this.mm = value;
				break;
		}
		this.value = this.recombineDateAndTime();
		this.propagateChange(this.value);
	}

	private splitIntoDateAndTime(value: string): void {
		const dateObj = new Date(value);
		const d = dateObj.getDate();
		const m = dateObj.getMonth();
		const y = dateObj.getFullYear();
		this.date = `${d}/${m + 1}/${y}`;
		this.hh = `0${dateObj.getHours().toString()}`.slice(-2);
		this.mm = `0${dateObj.getMinutes().toString()}`.slice(-2);
	}

	private recombineDateAndTime() {
		const [ d, m, y ] = this.date.split('/');
		return new Date(parseInt(y), parseInt(m) - 1, parseInt(d), parseInt(this.hh), parseInt(this.mm));
	}

}
