import { Component, forwardRef, ChangeDetectorRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { CustomInputComponent } from './../../_abstract/custom-input.component';
import { FriendlyValidationErrorsService } from './../../../services/friendly-validation-errors.service';

@Component({
	selector: 'app-multiline',
	templateUrl: './multiline.component.html',
	styleUrls: ['./multiline.component.scss'],
	providers: [
		{
			provide: NG_VALUE_ACCESSOR,
			useExisting: forwardRef(() => MultilineComponent),
			multi: true
		}
	]
})
export class MultilineComponent extends CustomInputComponent {

	exposeMetaInTemplate: string[] = ['label', 'maxLineLength'];

	label: string;
	linesArr: string[];
	value: string;
	maxLineLength: number;

	readonly componentName = 'MultilineComponent'; // For AOT compatibility, as class names don't survive minification

	constructor(
		protected valErrsService: FriendlyValidationErrorsService,
		protected _cdr: ChangeDetectorRef
	) {
		super(valErrsService, _cdr);
	}

	writeValue(value: any): void {
		this.value = value;
		this.splitIntoLines(value, this._meta.lines || 5);
	}

	trackByFn(index: any, item: any) {
		return index;
	}

	private updateValue(): void {
		this.value = this.recombineLines(this.linesArr);
		this.propagateChange(this.value);
	}

	private splitIntoLines(value: string, numLines: number): void {
		this.linesArr = this.control.value.split(/\n/);
		if (this.linesArr.length < numLines) {
			const shortfall = numLines - this.linesArr.length;
			this.linesArr = [...this.linesArr, ...Array(shortfall).fill('')];
		} else {
			this.linesArr = this.linesArr.slice(0, numLines);
		}
	}

	private recombineLines = (linesArr: string[]): string => {
		return linesArr.join('\n');
	}

}
