import { Injectable, Inject, Optional } from '@angular/core';
import { FRIENDLY_VALIDATION_ERRORS, friendlyValidationErrors } from './../config/validation-messages.config';
import { FriendlyValErrors } from './../interfaces';
import { path } from 'ramda';


@Injectable({
	providedIn: 'root'
})
export class FriendlyValidationErrorsService {

	fErrs: FriendlyValErrors;

	constructor(@Inject(FRIENDLY_VALIDATION_ERRORS) fErrs: FriendlyValErrors) {
		if (!this.fErrs) {
			this.fErrs = friendlyValidationErrors;
		}
	}

	getFriendly(errorType: string, data?: StringMap<string|number>): string {
		const friendlyError = this.fErrs[errorType];
		if (friendlyError) {
			const dataNotEmpty = typeof data === 'object' && data !== null && Object.keys(data).length;
			return dataNotEmpty ? this.processTemplateSubstitutions(friendlyError, data) : friendlyError;
		}
		return errorType;
	}

	// -----------------
	// Simple templating

	private processTemplateSubstitutions(urlTemplate: string, data: StringMap<string|number>): string {
		const tokens = this.getSubstitutionTokens(urlTemplate);
		const processedUrl = tokens.reduce(
			(str, token) => str.replace(new RegExp(`{{\\s${token}\\s+}}`), path(token.split('.'), data)),
			urlTemplate
		);
		return processedUrl;
	}

	private getSubstitutionTokens(urlTemplate: string): string[] {
		const resArr = [];
		let tempArr;
		const regex = new RegExp('{{\\s+(.*?)\\s+}}', 'g');
		while ((tempArr = regex.exec(urlTemplate)) !== null) {
			resArr.push(tempArr[1]);
		}
		return resArr;
	}

}
